function [decision,conf] = cep_template (cause,effect,methodpars,outfile)
% function [decision,conf] = cep_template (cause,effect,methodpars,outfile)
%
% Wrapper that runs a causal discovery method on the pair (cause,effect)
% and writes the results to outfile. It also returns its decision about 
% the most likely causal direction and its confidence value(s). Since
% usually there is no unique confidence estimate, a method may return
% several values.
%
% INPUT:   cause            true cause (Nx1)
%          effect           true effect (Nx1)
%          methodpars       parameters
%          outfile          filename for storing results
%
% OUTPUT:  decision         decision about causal direction
%                             1    means correct
%                             -1   means wrong
%                             0    means none
%                             NAN  means error
%          conf             array of confidence values that the decision is correct.
%                           The number and precise meaning of those values depends
%                           on the method. In general, the higher the confidence,
%                           the more certain a method is. It should be -Inf in case
%                           decision == NAN.
%
% Copyright (c) 2008-2014  Joris Mooij  <j.m.mooij@uva.nl>
% All rights reserved.  See the file LICENSE for license terms.

  if size(cause,2) ~= 1 || size(effect,2) ~= 1
    decision = nan;
    conf = -Inf;
  else
    % set default parameters
    %
    % if ~isfield(methodpars,'par')
    %   methodpars.par = 0;
    % end
    % ...

    % fit model in forwards direction
    %
    % forward.fit = ...
    % forward.conf = ...

    % fit model in backwards direction
    % 
    % backward.fit = ...
    % backward.conf = ...

    % take decision
    %
    % decision = 0;
    % if forward.conf > backward.conf
    %   decision = 1;
    % elseif forward.conf < backward.conf
    %   decision = -1;
    % end

    % calculate confidence
    % conf = max([forward.conf, backward.conf]);

    % save results
    % save(outfile,'cause','effect','methodpars','forward','backward','decision','conf','-v7.3');
  end

return
