/*  Copyright (c) 2009  Joris Mooij  <j.m.mooij@uva.nl>
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
*/


#include "mex.h"
#include <algorithm>
#include <vector>
#include <cmath>
#include <iostream>

/* Input Arguments */

#define N_IN       prhs[0]
#define SEED_IN    prhs[1]
#define NR_IN      1
#define NR_IN_OPT  1


/* Output Arguments */

#define PI_OUT     plhs[0]
#define NR_OUT     0
#define NR_OUT_OPT 1


void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray*prhs[] ) { 
    /* Check for proper number of arguments */
    if( ((nrhs < NR_IN) || (nrhs > NR_IN + NR_IN_OPT)) || ((nlhs < NR_OUT) || (nlhs > NR_OUT + NR_OUT_OPT)) ) { 
        mexErrMsgTxt("Usage: myrandperm(n[,seed])\n\n"
        "Returns a random permutation of the integers from 1 to n.\n\n"
        "INPUT:    n      = number of elements\n"
        "OPTIONAL: seed   = random number seed (integer)\n"
        "OUTPUT:          = random permutation of {1,2,...,n}\n"
        "\n"
        "Copyright (c) 2008-2010  Joris Mooij  <j.mooij@cs.ru.nl>\n"
        "All rights reserved.  See the file LICENSE for the license terms\n");
    } 

    if( !mxIsDouble(N_IN) || (mxGetM(N_IN) != 1) || (mxGetN(N_IN) != 1) )
        mexErrMsgTxt("n should be a scalar\n");
    int n = (int)round(*((double *)mxGetPr(N_IN)));
    if( n < 1 )
        mexErrMsgTxt("n should be >= 1\n");
    
    int seed = 12345;
    if( nrhs > NR_IN ) {
        if( !mxIsDouble(SEED_IN) || (mxGetM(SEED_IN) != 1) || (mxGetN(SEED_IN) != 1) )
            mexErrMsgTxt("seed should be a scalar\n");
        seed = (int)round(*((double *)mxGetPr(SEED_IN)));
    }

    PI_OUT = mxCreateDoubleMatrix(1,n,mxREAL);
    double *pi_out = mxGetPr(PI_OUT);

    // construct random permutation
    std::vector<size_t> permutation(n, 0);
    for( size_t i = 0; i < n; i++ )
        permutation[i] = i + 1;
    srand(seed);
    random_shuffle( permutation.begin(), permutation.end() );

    for( size_t i = 0; i < n; i++ )
        pi_out[i] = permutation[i];
    
    return;
}
